@@ -1,7 +1,15 @@
-priorformat<-function(prior, start, nfl, meta, diagR){
+priorformat<-function(prior, start, nfl, meta, diagR, vtype){
 
        if(is.null(prior)){
-          prior<-list(V=diag(sum(nfl)), nu=0, fix=as.numeric(meta), alpha.mu=rep(1,sum(nfl)), alpha.V=diag(sum(nfl))*0)
+          prior<-list(V=diag(sum(nfl)), nu=0, fix=as.numeric(meta), alpha.mu=rep(0,sum(nfl)), alpha.V=diag(sum(nfl))*0, beta.mu=NULL, beta.V=NULL, covu=FALSE)
+          if(grepl("ante", vtype[1])){
+            nk<-as.numeric(gsub("[a-z]", "", vtype[1]))
+            if(!grepl("antec", vtype[1])){
+              nk<-nk*(nfl-(nk+1)/2)
+            }
+            prior$beta.mu<-rep(0,nk)
+            prior$beta.V<-diag(nk)*1e+10
+          }
        }else{
          if(is.null(prior$V)){
            stop("V not specified for some prior$G/prior$R elements")
@@ -15,11 +23,36 @@ priorformat<-function(prior, start, nfl, meta, diagR){
          if(!is.null(prior$alpha.V) & diagR>0){
            stop("Parameter exapnded priors not implemented for residual structures")
          }
+         if(is.null(prior$covu)){
+           prior$covu<-FALSE
+         }else{
+           if(diagR<1){
+             stop("covu is only an argument for residual structures")
+           }else{
+             if(!is.logical(prior$covu)){stop("covu should be logical")}
+             if(prior$covu){
+               if(grepl("ante.*v$", vtype[1]) & prior$covu){stop("covu=TRUE cannot be used with antedependence structures")}
+             }
+           }
+         }
+         if(!grepl("ante", vtype[1])){
+           if(!is.null(prior$beta.mu)){
+             stop("beta.mu in prior specification only possible with antedependence structures")
+           }
+           if(!is.null(prior$beta.V)){
+             stop("beta.V in prior specification only possible with antedependence structures")
+           }
+         }
+         if(grepl("ante.*v$", vtype[1])){
+             if(nrow(prior$V)!=1){stop("prior$V should be a scalar with antev structures")}
+             prior$V<-diag(nfl)*as.numeric(prior$V)
+         }
          if(is.null(prior$fix)){
            prior$fix<-0
          }
-         if(any(names(prior)%in%c("V", "n", "nu", "alpha.mu", "alpha.V", "fix")==FALSE)){
-            paste(paste(names(prior)[which(names(prior)%in%c("V", "n", "nu", "alpha.mu", "alpha.V", "fix")==FALSE)], sep=" "), " are not valid prior specifications for G/R-structures")
+         pnames<-c("V", "n", "nu", "alpha.mu", "alpha.V", "fix", "beta.mu", "beta.V", "covu")
+         if(any(names(prior)%in%pnames==FALSE)){
+            paste(paste(names(prior)[which(names(prior)%in%pnames==FALSE)], sep=" "), " are not valid prior specifications for G/R-structures")
          }
          if(diagR==3){     # need to expand trait:units prior to us(trait):units prior      
            if(dim(prior$V)[1]!=1){
@@ -29,6 +62,7 @@ priorformat<-function(prior, start, nfl, meta, diagR){
          }else{
            if(any(dim(prior$V)!=sum(nfl))){
              stop("V is the wrong dimension for some prior$G/prior$R elements")
+             # check not performed for covu - checked outside of priorformat
            }
          }
          if(diagR==1){
@@ -54,7 +88,7 @@ priorformat<-function(prior, start, nfl, meta, diagR){
            }
          } 
          if(is.null(prior$alpha.mu)){
-            prior$alpha.mu<-matrix(1, nrow(prior$V), 1)
+            prior$alpha.mu<-matrix(0, nrow(prior$V), 1)
          }else{
            if(is.matrix(prior$alpha.mu)==FALSE){
              prior$alpha.mu<-matrix(prior$alpha.mu, length(prior$alpha.mu), 1)
@@ -62,7 +96,28 @@ priorformat<-function(prior, start, nfl, meta, diagR){
            if(length(prior$alpha.mu)!=nrow(prior$alpha.V)){
              stop("alpha.mu is the wrong length for some prior$G/prior$R elements")
            }
-         } 
+         }
+         if(grepl("ante", vtype[1])){
+           nk<-as.numeric(gsub("[a-z]", "", vtype[1]))
+           if(!grepl("antec", vtype[1])){
+             nk<-nk*(nfl-(nk+1)/2)
+           }
+           if(!is.null(prior$beta.mu)){
+             if(length(prior$beta.mu)!=nk){stop("beta.mu is the wrong length for some prior$G/prior$R elements")}
+           }else{
+             prior$beta.mu<-rep(0,nk)
+           }
+           if(!is.null(prior$beta.V)){
+             if(is.matrix(prior$beta.V)==FALSE){
+               prior$beta.V<-as.matrix(prior$beta.V)
+             }
+             if(nrow(prior$beta.V)!=ncol(prior$beta.V)){stop("beta.V is not square for some prior$G/prior$R elements")}
+             if(!is.positive.definite(prior$beta.V)){stop("beta.V is not positive-deifinite for some prior$G/prior$R elements")}
+             if(length(prior$beta.mu)!=nrow(prior$beta.V)){stop("beta.V is the wrong length for some prior$G/prior$R elements")}
+           }else{
+             prior$beta.V<-diag(nk)*1e+10
+           }
+         }
          if(prior$fix!=0){
            CM<-prior$V[prior$fix:nrow(prior$V),prior$fix:nrow(prior$V)]         
            if(prior$fix!=1){
@@ -87,8 +142,14 @@ priorformat<-function(prior, start, nfl, meta, diagR){
        }else{
          if(is.matrix(start)==FALSE){
            start<-as.matrix(start)
+         }
+         if(diagR==3){     # need to expand trait:units prior to us(trait):units prior   
+           if(dim(start)[1]!=1){
+             stop("V is the wrong dimension for some strart$G/start$R elements")
+           }
+           start<-diag(sum(nfl))*start[1]
          }	
-         if(any(dim(start)!=sum(nfl))){
+         if(any(dim(start)!=sum(nfl)) ){
            stop("V is the wrong dimension for some start$G/start$R elements")
          }
          if(is.positive.definite(start)==FALSE){
@@ -108,7 +169,7 @@ priorformat<-function(prior, start, nfl, meta, diagR){
          }
        }
 
-       prior<-mapply(x=cumsum(nfl)-(nfl-1), y=cumsum(nfl),  function(x,y){list(V=as.matrix(prior$V[x:y,x:y, drop=FALSE]), nu=prior$n, fix=pfix(x,y), alpha.mu=prior$alpha.mu[x:y], alpha.V=as.matrix(prior$alpha.V[x:y,x:y, drop=FALSE]))}, SIMPLIFY=FALSE)
+       prior<-mapply(x=cumsum(nfl)-(nfl-1), y=cumsum(nfl),  function(x,y){list(V=as.matrix(prior$V[x:y,x:y, drop=FALSE]), nu=prior$n, fix=pfix(x,y), alpha.mu=prior$alpha.mu[x:y], alpha.V=as.matrix(prior$alpha.V[x:y,x:y, drop=FALSE]), beta.mu=prior$beta.mu, beta.V=prior$beta.V, covu=prior$covu)}, SIMPLIFY=FALSE)
        start<-mapply(x=cumsum(nfl)-(nfl-1), y=cumsum(nfl),  function(x,y){list(start=as.matrix(start[x:y,x:y, drop=FALSE]))}, SIMPLIFY=FALSE)
 
        return(list(prior=prior, start=start))
